#ifndef stk500boot_h
#define stk500boot_h

/*
  these are used to test issues
  http://code.google.com/p/arduino/issues/detail?id=505
  Reported by mark.stubbs, Mar 14, 2011
  The STK500V2 bootloader is comparing the seqNum to 1 or the current sequence
  (IE: Requiring the sequence to be 1 or match seqNum before continuing).
  The correct behavior is for the STK500V2 to accept the PC's sequence number, and echo it back for the reply message.
*/
#define _FIX_ISSUE_505_

#ifndef EEWE
#define EEWE    1
#endif
#ifndef EEMWE
#define EEMWE   2
#endif

// HW and SW version, reported to AVRISP, must match version of AVRStudio
#define CONFIG_PARAM_BUILD_NUMBER_LOW 0
#define CONFIG_PARAM_BUILD_NUMBER_HIGH  0
#define CONFIG_PARAM_HW_VER       0x0F
#define CONFIG_PARAM_SW_MAJOR     2
#define CONFIG_PARAM_SW_MINOR     0x0A

/*
   Calculate the address where the bootloader starts from FLASHEND and BOOTSIZE
   (adjust BOOTSIZE below and BOOTLOADER_ADDRESS in Makefile if you want to change the size of the bootloader)
*/
//#define BOOTSIZE 1024
#if (FLASHEND > 0x0F000)
#define BOOTSIZE 8192
#else
#define BOOTSIZE 4096
#endif

#define APP_END  (FLASHEND -(2*BOOTSIZE) + 1)

// States used in the receive state machine
#define ST_START    0
#define ST_GET_SEQ_NUM  1
#define ST_MSG_SIZE_1 2
#define ST_MSG_SIZE_2 3
#define ST_GET_TOKEN  4
#define ST_GET_DATA   5
#define ST_GET_CHECK  6
#define ST_PROCESS    7


uint8_t processStk500boot(void);

#endif
